use enum_map::Enum;
use enum_ordinalize::Ordinalize;
use parse_display::{Display, FromStr};
use serde_with::{DeserializeFromStr, SerializeDisplay};

#[derive(Debug, Eq, PartialEq, Hash, Clone, Copy, Display, FromStr, SerializeDisplay, DeserializeFromStr)]
/// One-based coordinates of a specific virtual (not physical) key.
#[display("{x}x{y}")]
pub struct KeyPosition {
    pub x: u16,
    pub y: u16,
}

#[derive(Debug, Eq, PartialEq, Hash, Clone, Display, FromStr, SerializeDisplay, DeserializeFromStr)]
#[display("{page_id}/{position}")]
pub struct KeyPath {
    pub page_id: String,
    pub position: KeyPosition,
}

#[derive(Debug, Eq, PartialEq, Hash, Clone, Copy, Enum, Ordinalize, Display, FromStr, SerializeDisplay, DeserializeFromStr)]
#[display(style = "kebab-case")]
pub enum KnobPosition {
    LeftTop,
    LeftMiddle,
    LeftBottom,
    RightTop,
    RightMiddle,
    RightBottom,
}

impl KnobPosition {
    pub fn is_left(&self) -> bool {
        matches!(self, KnobPosition::LeftBottom | KnobPosition::LeftMiddle | KnobPosition::LeftTop)
    }
}

#[derive(Debug, Eq, PartialEq, Hash, Clone, Display, FromStr, SerializeDisplay, DeserializeFromStr)]
#[display("{page_id}/{position}")]
pub struct KnobPath {
    pub page_id: String,
    pub position: KnobPosition,
}
