use clap::Parser;
use color_eyre::Result;
use pa_volume_interface::PaVolumeInterface;
use std::sync::Arc;
use std::thread::sleep;
use std::time::Duration;

use crate::handler::Handler;

mod handler;

#[derive(Debug, Parser)]
#[command(name = "pa_volume")]
enum CliCommand {
    #[command(about = "Print all currently available entities")]
    Entities,
    #[command(name = "deckster-run", hide = true)]
    Run,
}

fn main() -> Result<()> {
    env_logger::init();
    let command = CliCommand::parse();

    match command {
        CliCommand::Entities => print_entities(),
        CliCommand::Run => {
            deckster_mode::run(Handler::new)?;
        }
    }

    Ok(())
}

fn print_entities() {
    let pa_volume_interface = Arc::new(PaVolumeInterface::spawn_thread("deckster handler".to_owned()));
    sleep(Duration::from_secs(1)); // wait for the data to load

    let full_state = pa_volume_interface.current_state();
    let entities_by_id = full_state.entities_by_id();

    for (id, state) in entities_by_id {
        println!("{id}: {:#?}\n", state.metadata());
    }
}
