use color_eyre::Result;
use tiny_skia::{ColorU8, Pixmap, PremultipliedColorU8};

use deckster_shared::image_filter::ImageFilterDestructive;

pub fn apply(original: &Pixmap, filter: &ImageFilterDestructive) -> Result<Pixmap> {
    let mut result = original.clone();

    for p in result.pixels_mut() {
        if filter.invert {
            *p = PremultipliedColorU8::from_rgba(p.alpha() - p.red(), p.alpha() - p.green(), p.alpha() - p.blue(), p.alpha()).unwrap();
        }

        if filter.grayscale {
            let a = p.alpha();
            if a > 0 {
                // The values are adjusted to make it look right on the Loupedeck Live display
                let luminosity = (0.3 * p.red() as f32 + 0.6 * p.green() as f32 + 0.1 * p.blue() as f32).floor() as u8;
                *p = ColorU8::from_rgba(luminosity, luminosity, luminosity, a).premultiply();
            }
        }

        if let Some(color) = &filter.color {
            let a = p.alpha();
            if a > 0 {
                *p = ColorU8::from_rgba(color.r, color.g, color.b, a).premultiply();
            }
        }
    }

    Ok(result)
}
